//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/H2OMapping.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * H2OMapping.h
 *
 *  Created on: 03.02.2012
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#ifndef H2OMAPPING_H_
#define H2OMAPPING_H_
#include "qImage.h"
#include "qRecon.h"
#include <boost/shared_ptr.hpp>
#include <list>

class qH2OMapping: public qRecon {

	std::list<boost::shared_ptr<qImage> > HighBCor;
	std::list<boost::shared_ptr<qImage> > M0;
	std::list<boost::shared_ptr<qImage> > T1Maps;
	std::list<boost::shared_ptr<qImage> > FlipMapCor;
	std::list<boost::shared_ptr<qImage> > CoilCor;
	std::list<boost::shared_ptr<qImage> > EPIBody;
	std::list<boost::shared_ptr<qImage> > EPIHead;
	std::list<boost::shared_ptr<qImage> > MaskLowRes;
	std::list<boost::shared_ptr<qImage> > RefSIMask;
	std::list<boost::shared_ptr<qImage> > H20Map;
	double FlipAngle;
	double TR;
	bool applyHighFieldCor;
	bool coilCorCalculated;

	void calcCoilCor();

public:

	qH2OMapping();
	virtual ~qH2OMapping();

	virtual bool run(void);
	void calculateHighFieldCorrection(
			std::list<boost::shared_ptr<qImage> > WMMask,
			std::list<boost::shared_ptr<qImage> > GMMask,
			std::list<boost::shared_ptr<qImage> > CSFMask,
			double WMMedian,
			double GMMedian,
			double CSFMedian);

	static std::list<boost::shared_ptr<qImage> > calculateEffectiveM0(
			std::list<boost::shared_ptr<qImage> > M0T2Star,
			std::list<boost::shared_ptr<qImage> > M0Mylein,
			std::list<boost::shared_ptr<qImage> > M0PolyFit,
			std::list<boost::shared_ptr<qImage> > T2StarMap);

	void addRelevantData(std::list<boost::shared_ptr<qImage> > T1Maps,
			std::list<boost::shared_ptr<qImage> > MO,
			std::list<boost::shared_ptr<qImage> > FlipMapCorrection,
			std::list<boost::shared_ptr<qImage> > ReferenceSignalMask,
			std::list<boost::shared_ptr<qImage> > EpiBody,
			std::list<boost::shared_ptr<qImage> > EpiHead,
			std::list<boost::shared_ptr<qImage> > MaskLowResolution,
			double FlipQuteLow,
			double TRQuteLow,
			int nSlices);

	std::list<boost::shared_ptr<qImage> > getH2OMap(void) {
		return H20Map;
	}

	std::list<boost::shared_ptr<qImage> > getHighFieldCor(void) {
		return HighBCor;
	}

	std::list<boost::shared_ptr<qImage> > getCoilCor(void) {
		return CoilCor;
	}

	std::list<boost::shared_ptr<qImage> > getEffectiveM0(void) {
		return M0;
	}

	void highFieldCorON()  { applyHighFieldCor=true; }
	void highFieldCorOFF() { applyHighFieldCor=false; }

};

#endif /* H2OMAPPING_H_ */
