//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/MyelinMapping.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * MyelinMapping.h
 *
 *  Created on: 12.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109
 */

#ifndef MYELINMAPPING_H_
#define MYELINMAPPING_H_

#include "qRecon.h"
#include <boost/numeric/ublas/matrix.hpp>


class qMyelinMapping: public qRecon {

public:

	typedef enum {
		IS_10_TE_DATASET = 0,
		IS_8_TE_DATASET = 1
	} WHICH_DATA;

	typedef enum {
		OPTIMISE_MYELIN = 0,
		OPTIMISE_M0 = 0,
	} OPTIMISE_WHAT;


private:
	std::list<boost::shared_ptr<qImage> > quteLowFlip;
	std::list<boost::shared_ptr<qImage> > T2StarSEMap;
	std::list<boost::shared_ptr<qImage> > MyelinMap;
	std::list<boost::shared_ptr<qImage> > T2SfastMap;
	std::list<boost::shared_ptr<qImage> > T2SslowMap;
	std::list<boost::shared_ptr<qImage> > M0;


	int nTE;
	vector<double> TE;
	float SNR;
	int NInnerLoosps;

	int resFacRows;
	int resFacCols;

	std::string simMatrixLocation;
	int T2StarMax;
	vector<int> T2StarFull;
	matrix<float> Afull;
	matrix<float> A1;
	int lengthFastRelaxInterval;
	std::string DebugOutFolder;

	// Reconstruction parameters
	WHICH_DATA dataSource;
	int mT2low;
	int wlow;
	double whigh;
	float lambda;
	double lowSmoothScale;
	double highBoundAsym;


	void estimateMyelin(const matrix<float>& simMatrix, const vector<float> y, float curT2StarSE, float& curMyelinEstimate,
						float& curT2SsEstimate, float& curT2SfEstimate, float& curM0Estimate, bool useInitialMyelinEstimate);

public:

	qMyelinMapping();
	qMyelinMapping(std::string MyelinMappingMatrixFile);
	virtual ~qMyelinMapping();

	virtual bool run(void);
	void SimTest(OPTIMISE_WHAT whatToOptimise);
	bool determineM0(void);

	void addRelevantData(std::list<boost::shared_ptr<qImage> > quteLowFlip,
						 std::list<boost::shared_ptr<qImage> > T2StarSEMap,
						 int nSlices,
						 float SNR);


	std::list<boost::shared_ptr<qImage> > getMyelinMap(void) { return MyelinMap; }
	std::list<boost::shared_ptr<qImage> > getT2StarFastMap(void) { return T2SfastMap; }
	std::list<boost::shared_ptr<qImage> > getT2StarSlowMap(void) { return T2SslowMap; }
	std::list<boost::shared_ptr<qImage> > getM0(void) { return M0; }
};

#endif /* MYELINMAPPING_H_ */
