//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/T1Mapping.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * T1Mapping.cpp
 *
 *  Created on: 24 oct. 2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program.

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "T1Mapping.h"
#include "qImageMap.h"
#include "qImageUtils.h"
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <list>
#include <algorithm>
#include <math.h>

qT1Mapping::qT1Mapping() {

	// TODO Auto-generated constructor stub

}

qT1Mapping::~qT1Mapping() {
	// TODO Auto-generated destructor stub
}

void qT1Mapping::addRelevantData(
		std::list<boost::shared_ptr<qImage> > QuteLowFlip,
		std::list<boost::shared_ptr<qImage> > QuteHighFlip,
		std::list<boost::shared_ptr<qImage> > EflipMap, int numSlices) {

	boost::shared_ptr<qImageMap> curImg;
	boost::numeric::ublas::vector<int> imageDimVec(3);
	std::list<boost::shared_ptr<qImage> >::iterator imageIter;
	std::list<boost::shared_ptr<qImage> > SliceListe;
	bool found = false;
	int count = 0;

	// assign member variables
	EfflipsMap = EflipMap;
	quteHighFlip = QuteHighFlip;
	nSlices = numSlices;
	quteLowFlip = QuteLowFlip;
	nTE = (int) (quteHighFlip.size() / nSlices);
	TE.resize(nTE, false);
	imageIter = QuteHighFlip.begin();
	nRow = (*imageIter)->getNrows();
	nCol = (*imageIter)->getNcols();

	SliceListe = qImageUtils::getImagesFromSlice(quteHighFlip, 1);
	if (SliceListe.empty() || (int) SliceListe.size() != nTE)
		throw "T1Mapping::addRelevantData : Illegal number of images in the first slice of QuteHighFlip :"
				+ (int) SliceListe.size();
	for (imageIter = SliceListe.begin(); imageIter != SliceListe.end();
			imageIter++)
		TE(count++) = boost::any_cast<double>(
				(*imageIter)->getHeader().getHeaderParam("EchoTime", &found));

	// get relevant dicom header parameter
	m_TR1 = boost::any_cast<double>( (*(quteLowFlip.begin()) )->getHeader().getHeaderParam("RepetitionTime", &found) );
	m_alpha1 = boost::any_cast<double>( (*(quteLowFlip.begin()) )->getHeader().getHeaderParam("FlipAngle", &found) );
	m_TR2 = boost::any_cast<double>( (*(quteHighFlip.begin()) )->getHeader().getHeaderParam("RepetitionTime", &found) );
	m_alpha2 = boost::any_cast<double>( (*(quteHighFlip.begin()) )->getHeader().getHeaderParam("FlipAngle", &found) );


	// create result image objects if not yet created
	if (T1Map.empty() || (int) T1Map.size() != nSlices) {
		// clean up first if necessary
		T1Map.erase(T1Map.begin(), T1Map.end());

		for (int slice = 1; slice <= nSlices; slice++) {
			imageDimVec(2) = slice;
			curImg = boost::shared_ptr<qImageMap>(
					new qImageMap(imageDimVec, nRow, nCol));
			T1Map.push_back(curImg);

		}

	}

}

// assign member variables
bool qT1Mapping::run(void) {

	std::list<boost::shared_ptr<qImage> >::iterator greLowIter;
	std::list<boost::shared_ptr<qImage> > greHighFlipList;
	std::list<boost::shared_ptr<qImage> > greLowFlipList;
	std::list<boost::shared_ptr<qImageBinMask> >::iterator maskIter;
	std::list<boost::shared_ptr<qImage> >::iterator T1Iter;
	std::list<boost::shared_ptr<qImage> >::iterator greHighIter;
	std::list<boost::shared_ptr<qImage> >::iterator FlipIter;

	vector<double> y(nTE);
	int nT1 = 3800;
	int minT1 = 200;
	vector<double> T1_vek(nT1, 0);
	matrix<double> SI(180, nT1);
	vector<double>::iterator it;
	double result1;
	double curGreHigh=0;
	double curGreLow=0;
	double curSquaredDiff=0;
	double min = 0;
	int minIdx = 0;
	double minT1_1300 = 0;
	double minT1_1301 =0;
	int	startT1=minT1;
	int	endT1=nT1;

	double A;
	double B;
	double C;
	double D;
	int count;
	int slice = 0;
	double result;

	for (int i = 0; i < 180; i++) {
		for (int m = 0; m < nT1; m++) {
			// int n=i-1;
			// 0.0175=PI/180(convert degrees to radians)
			A = 1 - exp(-(m_TR1 / m));
			D = 1 - exp(-(m_TR2 / m));
			B = 1 - cos(m_alpha1 * i / 90.0) * exp(-(m_TR1 / m));
			C = 1 - cos(m_alpha2 * i / 90.0) * exp(-(m_TR2 / m));
			double a = A / B * sin(m_alpha1 * i / 90.0);
			double b = D / C * sin(m_alpha2 * i / 90.0);
			if (b > 0) {
				SI(i, m) = a / b;
			}
		}
	}

	// consistency check
	if (T1Map.empty()) {
		throw "T1Mapping::run : no data were provided for mapping. Run 'addRelevantData()' first";
	}

	// Loop over all slices
	for (slice = 1; slice <= nSlices; slice++) {
		//for(slice1=1; slice1<=nSlices; slice1++){

		// set iterators properly (required for parallel processing)
		T1Iter = T1Map.begin();
		std::advance(T1Iter, slice - 1);
		maskIter = mask.begin();
		std::advance(maskIter, slice - 1);
		FlipIter = EfflipsMap.begin();
		std::advance(FlipIter, slice - 1);

		// get all images (timepoints) for current slice
		greHighFlipList = qImageUtils::getImagesFromSlice(quteHighFlip, slice);
		greLowFlipList = qImageUtils::getImagesFromSlice(quteLowFlip, slice);
		matrix<int>& FlipMap = (*FlipIter)->getPixelMatrix();
		matrix<int>& maskMat = (*maskIter)->getPixelMatrix();

		// loop over all points in matrix
		for (int row = 0; row < nRow; row++) {
			for (int col = 0; col < nCol; col++) {

				// is current pixel contained in image mask?
				if (maskMat(row, col) == 0) {
					(*T1Iter)->setPixel(row, col, 0.0);
					//(*M0Iter)->setPixel(row,col,0.0);
					continue;
				}

				// get current effective flip angle for a nominal value of 90
				int curFlip = std::min(179, (int) (90.0 * FlipMap(row, col)/1000.0));

				// fill y-vector
				count = 0;
				greLowIter = greLowFlipList.begin();
				for (greHighIter = greHighFlipList.begin(); greHighIter != greHighFlipList.end();
						greHighIter++) {

					curGreLow =  (*greLowIter)->getPixel(row,col);
					curGreHigh =  (*greHighIter)->getPixel(row,col);
					if (curGreHigh > 0)
						y(count++) = curGreLow/curGreHigh;

					greLowIter++;
				}

				result = 0;
				for (it = y.begin(); it != y.end(); it++) {

					// Restrict search range (remember: function has a global minimum!)
					minT1_1300 = pow(SI(curFlip, 1300) - *it, 2);;
					minT1_1301 = pow(SI(curFlip, 1301) - *it, 2);;
					if( minT1_1300 > minT1_1301) {
						startT1=1301;
						endT1=nT1;
					}
					else {
						startT1=minT1;
						endT1=1300;
					}

					min = pow(SI(curFlip, startT1) - *it, 2);;
					minIdx = startT1;

					for (int j = startT1+1; j < endT1; j++) {
						curSquaredDiff = pow(SI(curFlip, j)-*it, 2);
						if( curSquaredDiff >= min )
							break; 	// here we can break as the functions has only one global minimum
						min=curSquaredDiff;
						minIdx=j;
					}

					result = (result + minIdx);
				}

				result1 = result / greHighFlipList.size();
				(*T1Iter)->setPixel(row, col, (int) (result1));
			}
		}
	}

	return true;
}
