//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/T1Mapping.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * T1Mapping.h
 *
 *  Created on: 24 oct. 2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#ifndef T1MAPPING_H_
#define T1MAPPING_H_
#include "qImage.h"
#include "qRecon.h"
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <list>

class qT1Mapping: public qRecon {
	std::list<boost::shared_ptr<qImage> > quteLowFlip;
	std::list<boost::shared_ptr<qImage> > quteHighFlip;
	std::list<boost::shared_ptr<qImage> > T1Map;
	std::list<boost::shared_ptr<qImage> > EfflipsMap;

	double m_TR1;
	double m_TR2;
	double m_alpha1;
	double m_alpha2;
	int nTE;
	vector<double> TE;

public:
	qT1Mapping();
	virtual ~qT1Mapping();
	virtual bool run(void);
	void addRelevantData(std::list<boost::shared_ptr<qImage> > QuteLowFlip,
			std::list<boost::shared_ptr<qImage> > QuteHighFlip,
			std::list<boost::shared_ptr<qImage> > EflipMap,
			int numSlices);

	std::list<boost::shared_ptr<qImage> > getT1Map(void) {
		return T1Map;
	}
};

#endif /* T1MAPPING_H_ */
