//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/efflipMap.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * efflipMap.cpp
 *
 *  Created on: 7 dc. 2011
 *      Author: neeb
 *
	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "efflipMap.h"
#include "qImageMap.h"
#include "qImageUtils.h"
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <list>
#include <algorithm>
#include <math.h>
#include <float.h>

efflipMap::efflipMap() {
	// TODO Auto-generated constructor stub

}

efflipMap::~efflipMap() {
	// TODO Auto-generated destructor stub
}

void efflipMap::addRelevantData(
		std::list<boost::shared_ptr<qImage> > EpiLowFlip,
		std::list<boost::shared_ptr<qImage> > EpiHighFlip, int numSlices) {

	boost::shared_ptr<qImageMap> curImg;
	boost::numeric::ublas::vector<int> imageDimVec(3);
	std::list<boost::shared_ptr<qImage> >::iterator imageIter;
	std::list<boost::shared_ptr<qImage> > SliceListEpiHigh;

	epiHighFlip = EpiHighFlip;
	nSlices = numSlices;
	epiLowFlip = EpiLowFlip;
	imageIter = epiHighFlip.begin();
	nRow = (*imageIter)->getNrows();
	nCol = (*imageIter)->getNcols();


	// ToDo: Check in EPI_LOW and EPI_HIGH headers, if flip angles are correct

	if (EflipMap.empty() || (int) EflipMap.size() != nSlices) {
		// clean up first if necessary
		EflipMap.erase(EflipMap.begin(), EflipMap.end());

		for (int slice = 1; slice <= nSlices; slice++) {
			imageDimVec(2) = slice;
			curImg = boost::shared_ptr<qImageMap>(
					new qImageMap(imageDimVec, nRow, nCol));
			EflipMap.push_back(curImg);
		}

	}
}

bool efflipMap::run(void) {

	std::list<boost::shared_ptr<qImage> >::iterator iterLowFlip;
	std::list<boost::shared_ptr<qImageBinMask> >::iterator maskIter;
	std::list<boost::shared_ptr<qImage> >::iterator iterFlipMap;
	std::list<boost::shared_ptr<qImage> >::iterator iterHighFlip;

	int slice = 0;
	double z = 0;
	double result1 = 0;


	if (EflipMap.empty()) {
		throw "efflipMap::run() : no data were provided for mapping. Run 'addRelevantData()' first";
	}

	// Loop over all slices
	for (slice = 1; slice <= nSlices; slice++) {

		// set iterators properly (required for parallel processing)
		iterFlipMap = EflipMap.begin();
		std::advance(iterFlipMap, slice - 1);
		maskIter = mask.begin();
		std::advance(maskIter, slice - 1);
		iterLowFlip = epiLowFlip.begin();
		std::advance(iterLowFlip, slice - 1);
		iterHighFlip = epiHighFlip.begin();
		std::advance(iterHighFlip, slice - 1);

		// Get matrices
		matrix<int>& maskMat = (*maskIter)->getPixelMatrix();
		matrix<int>& curMat = (*iterHighFlip)->getPixelMatrix();
		matrix<int>& curMat1 = (*iterLowFlip)->getPixelMatrix();

		// loop over all points in matrix
		for (int row = 0; row < nRow; row++) {
			for (int col = 0; col < nCol; col++) {

				// is current pixel contained in image mask?
				if (maskMat(row, col) == 0) {
					(*iterFlipMap)->setPixel(row, col, 0.0);
					continue;
				}

				// Calculate effective flip angle for current point
				if (curMat(row, col) > 0) {
					z = curMat(row, col) / (double) curMat1(row, col);
					result1 = 3.0 / 2.0 * acos((z - 1) / 2.0) / (PI/2) * 1000;
					if (std::isnan(result1) || std::isinf(result1)) {
						result1 = 0;
					}
					(*iterFlipMap)->setPixel(row, col, (int) (result1*0.9) );
				}
			}
		}
	}

	return true;
}

