/*============================================================================
// Name        : predictLib.cpp
// Author      : Neeb
// Version     :
// Copyright   : GPL
// Description : main test routine
// File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/predictLib.cpp $
// Revision: $Revision: 54 $
// Last modified: $Date: 2012-06-01 09:34:24 +0200 (Fr, 01 Jun 2012) $
// Last modified by: $Author: neeb $
//============================================================================
	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

*/

// ToDo's
// NIFTI export : http://nifti.nimh.nih.gov/nifti-1/documentation/faq
//                 http://afni.nimh.nih.gov/pub/dist/src/nifti/niftilib/
// Interpolation of flip maps over ventricles (might sometimes help for an improved estimate of ref-SI in H2O mapping)
// Field dependent segmentation cutoffs
// Version String in DICOM Header schreiben

#include <iostream>
#include <list>
#include <time.h>
#include <string>
#include <fstream>
#include <vector>
#include <boost/numeric/ublas/io.hpp>

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <boost/shared_ptr.hpp>
#include "uquadprog.hpp"
#include "qImage.h"
#include "qImageRaw.h"
#include "qImageMap.h"
#include "qImageHeader.h"
#include "qImageUtils.h"
#include "qImageProcessing.h"
#include "T2StarMapping.h"
#include "qublasUtils.h"
#include "MyelinMapping.h"
#include "T1Mapping.h"
#include "H2OMapping.h"
#include "efflipMap.h"
#include "qB0Mapping.h"
#include "qIOUtils.h"
#include "qImageStats.h"
#include "qSynthetic.h"
#include "qFSLProcessing.h"


int processDataSet(std::string FolderMain,std::ofstream& GlobalInfoFile,
		std::ofstream& GlobalResFile, std::string MyelinMappingMatrix);

int main(int argc, char** argv) {

	std::ofstream GlobalResFile;
	std::ofstream GlobalInfoFile;

	// Check arguments
	if( argc < 3 ) {
		std::cout << "Please provide at least the list with datasets to process and the myelin mapping matrix file." << std::endl;
		std::cout << "Call: pLib <Dataset List File> <Myelin Mapping Matrix File> <Global Info File> <Global Result File>" << std::endl;
		std::cout << "Exiting." << std::endl;
		return -1;
	}

	std::cout << "predictMS  Copyright (C) 2012 Heiko Neeb (RheinAhrCampus Remagen, University of Applied Sciences Koblenz)" <<std::endl;
	std::cout << "    This program comes with ABSOLUTELY NO WARRANTY." << std::endl;
	std::cout << "    This is free software, and you are welcome to redistribute it under certain conditions." << std::endl;
	std::cout << "    See the GNU General Public License for more details." << std::endl;

	if( argc > 3 )
		GlobalInfoFile.open (argv[3]);
	if( argc > 4 )
		GlobalResFile.open (argv[4]);

	char *BatchFile = argv[1];
	std::string MyelinMappingMatrix = argv[2];

	std::ifstream ifs( BatchFile );
	std::string FolderMain;
	std::vector<std::string> PatientList;
	time_t now;

	while( std::getline( ifs, FolderMain ) ) {
		if( FolderMain.compare(0,1,"%") == 0 ) {
			if( argc > 3) GlobalInfoFile << "Ignoring: "<< FolderMain.erase(0,1) <<std::endl;
		}
		else
			PatientList.push_back( FolderMain );
	}

	if( argc > 3) {
		GlobalInfoFile << std::endl ;
		GlobalInfoFile << "Processing the following " << PatientList.size() << " data set(s)" << std::endl;
		for( int i=0; i< (int)PatientList.size(); i++ ) {
			GlobalInfoFile << "Id="<< i+1<<" = " << PatientList[i] << std::endl;
		}

		time(&now);
		GlobalInfoFile << std::endl ;
		GlobalInfoFile << "Full reconstruction script started at: " << asctime(localtime(&now)) << std::endl;
	}


	for( int i=0; i< (int)PatientList.size(); i++ ) {
	//for( int i=0; i< 0; i++ ) {
		if( argc > 3 ) {
			GlobalInfoFile << std::endl << "________________________________________________________" << std::endl;
			GlobalInfoFile << "Processing : " << PatientList[i] << std::endl;
		}

		try {
			if( processDataSet(PatientList[i], GlobalInfoFile, GlobalResFile, MyelinMappingMatrix ) != 0 )
				return -1;
		}
		catch(std::string s)
		{
			std::cout  << s << std::endl;
			return -1;
		}
		catch(...) { // catch all exceptions
			return -1;
		}


		if( argc>3 ) GlobalInfoFile << std::endl;
	}

	time(&now);
	if( argc>3) GlobalInfoFile << "Full reconstruction script finished at: " << asctime(localtime(&now)) << std::endl;


#pragma omp parallel for
//	for( int i=0; i< (int)PatientList.size(); i++ ) {
	for( int i=0; i< 0; i++ ) {
#pragma omp critical
		{
			std::cout << std::endl << "________________________________________________________" << std::endl;
			std::cout << "FSL Processing of: " << PatientList[i] << std::endl;
		}

		qFSLProcessing::startFSLBatch(PatientList[i]);
	}


	// close streams
	if( argc > 3 )
		GlobalInfoFile.close();
	if( argc > 4 )
		GlobalResFile.close();

	return 0;
}



int processDataSet(std::string FolderMain, 	std::ofstream& GlobalInfoFile, std::ofstream& GlobalResFile,
		std::string MyelinMappingMatrix) {

	bool dumpGlobalRes=GlobalResFile.is_open();
	bool dumpGlobalInfo=GlobalInfoFile.is_open();
	int nSlices = 50;

	double TRGRELow;
	double FlipAngleGRELow;
	std::string PatientName;
	std::string PatientGender;
	std::string PatientAge;
	bool loadT1=false;
	bool loadH2O=false;
	bool loadMyelin=false;
	bool loadM0Myelin=false;
	bool highFieldCorrection=true;

	FolderMain.erase(std::remove(FolderMain.begin(), FolderMain.end(), '\r'), FolderMain.end());

//	std::string MyelinMappingMatrix = "/Users/neeb/Summary/Programming/predict/DataForMyelinMapping/";

	std::string FolderLow = FolderMain + "gre_lowflip/";
	std::string FolderHigh = FolderMain + "gre_highflip/";
	std::string FolderPhase = FolderMain + "gre_lowflip_phase/";
	std::string FolderEpiLow = FolderMain + "epi_lowflip/";
	std::string FolderEpiHigh = FolderMain + "epi_highflip/";
	std::string FolderEpiBody = FolderMain + "epi_body/";
	std::string FolderH2OMaps = FolderMain + "H2OMaps/";
	std::string FolderT1Maps = FolderMain + "T1Maps/";
	std::string FolderT2Maps = FolderMain + "T2Maps/";

	std::string FolderB0MapsMain = FolderMain + "B0Effective/";
	std::string FolderB0Maps = FolderB0MapsMain + "B0Effective/";
	std::string FolderGradientMaps = FolderB0MapsMain + "B0Gradient/";

	std::string FolderM0MapsMain = FolderMain + "M0/";
	std::string FolderM0MapsMyelin = FolderM0MapsMain + "M0Myelin/";
	std::string FolderM0MapsT2 = FolderM0MapsMain + "M0T2/";
	std::string FolderM0MapsEffective = FolderM0MapsMain + "M0Effective/";

	std::string FolderCorMapsMain = FolderMain + "Corrections/";
	std::string FolderCoilCor = FolderCorMapsMain + "CoilCor/";
	std::string FolderHighBCor = FolderCorMapsMain + "HighBCor/";

	std::string FolderMasks = FolderMain + "MasksForMapping/";
	std::string FolderBrainMask = FolderMasks + "BrainMask/";
	std::string FolderWMMask = FolderMasks + "WMMask/";
	std::string FolderGMMask = FolderMasks + "GMMask/";
	std::string FolderCSFMask = FolderMasks + "CSFMask/";

	std::string FolderMyelinMain = FolderMain + "Myelin/";
	std::string FolderMyelinMaps = FolderMyelinMain + "MyelinMaps/";
	std::string FolderMyelinMapsSmooth = FolderMyelinMain + "MyelinMapsSmooth/";
	std::string FolderT2StarSMaps = FolderMyelinMain + "T2Star_Slow/";
	std::string FolderT2StarFMaps = FolderMyelinMain + "T2Star_Fast/";

	std::string FolderSynthetic = FolderMain + "Synthetic/";
	std::string FolderSyntheticT1wGE = FolderSynthetic + "T1wGradEcho/";


	std::string FolderFlipMaps = FolderMain + "FlipMaps/";
	std::string filename;
	std::string dirname;
	time_t now;
	std::list<std::string> dirList;
	std::list<boost::shared_ptr<qImage> > greLowFlip;
	std::list<boost::shared_ptr<qImage> > greLowFlipPhase;
	std::list<boost::shared_ptr<qImage> > greHighFlip;
	std::list<boost::shared_ptr<qImage> > epiLow;
	std::list<boost::shared_ptr<qImage> > epiHigh;
	std::list<boost::shared_ptr<qImage> > epiBody;
	std::list<boost::shared_ptr<qImage> > tmpImgStore;
	std::list<boost::shared_ptr<qImage> > T2StarMap;
	std::list<boost::shared_ptr<qImage> > M0MapT2;
	std::list<boost::shared_ptr<qImage> > M0MapMyelin;
	std::list<boost::shared_ptr<qImage> > M0Map;
	std::list<boost::shared_ptr<qImage> > B0Map;
	std::list<boost::shared_ptr<qImage> > GradientMap;
	std::list<boost::shared_ptr<qImage> > H2OMap;
	std::list<boost::shared_ptr<qImage> > MyelinMap;
	std::list<boost::shared_ptr<qImage> > T2StarSlowMap;
	std::list<boost::shared_ptr<qImage> > T2StarFastMap;
	std::list<boost::shared_ptr<qImage> > FlipMap;
	std::list<boost::shared_ptr<qImage> > T1Map;
	std::list<boost::shared_ptr<qImage> > SNRMask;
	std::list<boost::shared_ptr<qImage> > SNRMaskLowRes;
	std::list<boost::shared_ptr<qImage> > BrainMask;
	std::list<boost::shared_ptr<qImage> > WMMask;
	std::list<boost::shared_ptr<qImage> > GMMask;
	std::list<boost::shared_ptr<qImage> > CSFMask;
	std::list<boost::shared_ptr<qImage> > tmpImageList;
	std::list<boost::shared_ptr<qImage> >::iterator imageIter;
	std::list<boost::shared_ptr<qImage> >::iterator maskIter;
	boost::shared_ptr<qImage> curImg;
	boost::numeric::ublas::vector<int> imageDimVec(4);
	qT2StarMapping T2Mapping;
	qMyelinMapping MyelinRecon(MyelinMappingMatrix);
	qT1Mapping T1Recon;
	qH2OMapping H2ORecon;
	efflipMap FlipMapping;
	qB0Mapping B0Mapping;

	time(&now);
	if (dumpGlobalInfo) GlobalInfoFile << "Patient recon started at: "<< asctime(localtime(&now)) << std::endl;

	////////////////////////////////////////////////////////////////////////////////////////////
	// import raw image data
	if (!qImage::importImageFolder<qImageRaw>(FolderLow, ".ima", nSlices,
			greLowFlip))
		return -1;

	if (!qImage::importImageFolder<qImageRaw>(FolderPhase, ".ima", nSlices,
			greLowFlipPhase))
		return -1;

	if (!qImage::importImageFolder<qImageRaw>(FolderHigh, ".ima", nSlices,
			greHighFlip))
		return -1;

	if (!qImage::importImageFolder<qImageRaw>(FolderEpiLow, ".ima", nSlices,
			epiLow))
		return -1;

	if (!qImage::importImageFolder<qImageRaw>(FolderEpiHigh, ".ima", nSlices,
			epiHigh))
		return -1;

	if (!qImage::importImageFolder<qImageRaw>(FolderEpiBody, ".ima", nSlices,
			epiBody))
		return -1;



	////////////////////////////////////////////////////////////////////////////////////////////
	// create folders for the reconstructed maps
	std::string cmd;
	if( !loadT1 )
		if( !qIOUtils::create_empty_directory( FolderT1Maps ) ) return -1;
	if( !loadH2O ) {
		if( !qIOUtils::create_empty_directory( FolderCorMapsMain ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderH2OMaps ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderCoilCor ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderHighBCor ) ) return -1;
	}
	if( !loadMyelin ) {
		if( !qIOUtils::create_empty_directory( FolderMyelinMain ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderMyelinMaps ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderMyelinMapsSmooth ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderT2StarSMaps ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderT2StarFMaps ) ) return -1;
	}

	if( !loadM0Myelin) {
		if( !qIOUtils::create_empty_directory( FolderM0MapsMain ) ) return -1;
		if( !qIOUtils::create_empty_directory( FolderM0MapsMyelin ) ) return -1;
	}

	if( !qIOUtils::create_empty_directory( FolderM0MapsT2 ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderM0MapsEffective ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderT2Maps ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderFlipMaps ) ) return -1;

	if( !qIOUtils::create_empty_directory( FolderB0MapsMain ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderB0Maps ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderGradientMaps ) ) return -1;

	if( !qIOUtils::create_empty_directory( FolderMasks ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderBrainMask ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderWMMask ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderGMMask ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderCSFMask ) ) return -1;

	if( !qIOUtils::create_empty_directory( FolderSynthetic ) ) return -1;
	if( !qIOUtils::create_empty_directory( FolderSyntheticT1wGE ) ) return -1;


	// determine image resolution(s) and get DICOM Header Info
	int nRow = (*greHighFlip.begin())->getNrows();
	int nCol = (*greHighFlip.begin())->getNcols();
	int nColEPI = (*epiHigh.begin())->getNcols();
	int nRowEPI = (*epiHigh.begin())->getNrows();

	bool found=true;
	qImageHeader& imgHeader = (*(greLowFlip.begin()) )->getHeader();
	TRGRELow = boost::any_cast<double>( imgHeader.getHeaderParam("RepetitionTime", &found) );
	FlipAngleGRELow = boost::any_cast<double>( imgHeader.getHeaderParam("FlipAngle", &found) );
	PatientName = boost::any_cast<std::string>( imgHeader.getHeaderParam("PatientName", &found) );
	PatientGender = boost::any_cast<std::string>( imgHeader.getHeaderParam("PatientGender", &found) );
	PatientAge = boost::any_cast<std::string>( imgHeader.getHeaderParam("PatientAge", &found) );


	// cut off first "nSlice" images in order to determine the SNR mask
	imageIter = greLowFlip.begin();
	std::advance(imageIter, nSlices);
	tmpImgStore.assign(greLowFlip.begin(), imageIter);

	// determine SNR masks (for both low and high resolution images)
	double SNR  = qImageUtils::createSNRMask(12.0, tmpImgStore, SNRMask);
	SNRMaskLowRes = qImageUtils::resize<qImageBinMask>(nColEPI, nRowEPI, SNRMask);

	// add mask to all relevant reconstruction algorithms
	T2Mapping.addMask(SNRMask);
	B0Mapping.addMask(SNRMask);
	MyelinRecon.addMask(SNRMask);
	T1Recon.addMask(SNRMask);
	H2ORecon.addMask(SNRMask);
	FlipMapping.addMask(SNRMaskLowRes);

/*
	// Test FFT  ... might need it in future for motian correction
	std::list<boost::shared_ptr<qImage> > FFTImages;
	for(int slice=1; slice<=400; slice++) {

		std::cout << "FFT Slice "<<slice <<std::endl;
		imageIter = greLowFlip.begin();
		std::advance(imageIter,slice-1);
		std::list<boost::shared_ptr<qImage> >::iterator phaseIter = greLowFlipPhase.begin();
		std::advance(phaseIter,slice-1);

		boost::shared_ptr<qImage> imageAbs= boost::shared_ptr<qImage>(new qImage(nRow, nCol));
		boost::shared_ptr<qImage> imagePhase= boost::shared_ptr<qImage>(new qImage(nRow, nCol));

		// FFT
		qublasUtils::FFT2D(**imageIter, **phaseIter, *imageAbs, *imagePhase);

		FFTImages.push_back(imageAbs);
	}

	if( !qImage::exportImageFolder( FolderT2Maps, "T2StarMap_Slice", ".ima", FFTImages, FolderLow) )
		return 1;

	return 0;*/


	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare T2* reconstruction
	if (dumpGlobalInfo) GlobalInfoFile << "Start T2* processing ..." <<std::endl;
	greLowFlip.sort(sort_by_slice);
	T2Mapping.addRelevantData(greLowFlip, nSlices);

	// Run T2* reconstruction
	if (!T2Mapping.run()) {
		std::cout << "T2* Reconstruction Failed!" << std::endl;
		std::cout << T2Mapping.getErrorMsg() << std::endl;
		exit(-1);
	}

	T2StarMap = T2Mapping.getT2StarMap();

	// Replace date in GRE lowflip (phase) if slice-slice artifact is observed
/*	qImageProcessing::correctGRELowFlipArtifact(greLowFlip, T2StarMap, SNRMask);

	// Re-Run T2* reconstruction
	if (!T2Mapping.run()) {
		std::cout << "T2* Reconstruction Failed!" << std::endl;
		std::cout << T2Mapping.getErrorMsg() << std::endl;
		exit(-1);
	}
	T2StarMap = T2Mapping.getT2StarMap();*/

	M0MapT2 = T2Mapping.getM0();
	if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;
	if( !qImage::exportImageFolder( FolderT2Maps, "T2StarMap_Slice", ".ima", T2StarMap, FolderHigh) )
		return 1;
	if( !qImage::exportImageFolder( FolderM0MapsT2, "M0(T2)_Slice", ".ima", M0MapT2, FolderHigh) )
		return 1;



	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare M0 reconstruction (from multiexponential analysis)
	if( loadM0Myelin ) {
		if (!qImage::importImageFolder<qImageRaw>(FolderM0MapsMyelin, ".ima", nSlices,
				M0MapMyelin))
			return -1;
	}
	else {
		if (dumpGlobalInfo) GlobalInfoFile << "Start M0 Reconstruction from multiexponential analysis ..."<<std::endl;
		MyelinRecon.addRelevantData(greLowFlip, T2StarMap, nSlices, SNR);
		if (!MyelinRecon.determineM0()) {
			std::cout << "M0 Reconstruction Failed!" << std::endl;
			std::cout << MyelinRecon.getErrorMsg() << std::endl;
			exit(-1);
		}

		if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;

		M0MapMyelin = MyelinRecon.getM0();
		if( !qImage::exportImageFolder( FolderM0MapsMyelin, "M0(Myelin)_Slice", ".ima", M0MapMyelin, FolderHigh) )
			return 1;

	}

	// calculate effective (resulting) M0
	M0Map = qH2OMapping::calculateEffectiveM0(M0MapT2, M0MapMyelin, tmpImgStore, T2StarMap);
	if( !qImage::exportImageFolder( FolderM0MapsEffective, "M0_Slice", ".ima", M0Map, FolderHigh) )
		return 1;



	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare B0 Field Mapping
	if (dumpGlobalInfo) GlobalInfoFile << "Start B0Field mapping ..." <<std::endl;
	greLowFlipPhase.sort(sort_by_slice);
	B0Mapping.addRelevantData(greLowFlipPhase, nSlices);

	// Run field mapping
	if (!B0Mapping.run()) {
		std::cout << "B0 Field-Mapping Failed!" << std::endl;
		std::cout << B0Mapping.getErrorMsg() << std::endl;
		exit(-1);
	}

	B0Map = B0Mapping.getB0Eff();
	GradientMap = B0Mapping.getGradient();

	if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;
	if( !qImage::exportImageFolder( FolderB0Maps, "B0Eff_Slice", ".ima", B0Map, FolderHigh) )
		return 1;
	if( !qImage::exportImageFolder( FolderGradientMaps, "Gradient_Slice", ".ima", GradientMap, FolderHigh) )
		return 1;


	////////////////////////////////////////////////////////////////////////////////////////////
	// FlipAngle reconstruction
	if (dumpGlobalInfo) GlobalInfoFile << "Start effective Flip Angle Reconstruction ..."<<std::endl;

	epiLow.sort(sort_by_slice);
	epiHigh.sort(sort_by_slice);
	FlipMapping.addRelevantData(epiLow, epiHigh, nSlices);

	if (!FlipMapping.run()) {
		std::cout << "FlipMap Reconstruction Failed!" << std::endl;
		std::cout << FlipMapping.getErrorMsg() << std::endl;
		exit(-1);
	}

	// Filter Flip Map
	FilterKernel flipMapKern;
	flipMapKern.createMeanKernel(5,5,5);
	tmpImageList = FlipMapping.getEflipNewMap();
	qImageProcessing::filterStackWithKernel(tmpImageList,flipMapKern, qImageProcessing::OMIT_ZEROS, qImageProcessing::KERNEL_FILTER);
	qImageProcessing::filterStackWithKernel(tmpImageList,flipMapKern, qImageProcessing::FILL_ZEROS_ONLY,  qImageProcessing::KERNEL_FILTER);

	// Reshape and re-filter flip map
	FlipMap = qImageUtils::resize<qImageMap>(nCol, nRow, FlipMapping.getEflipNewMap());
	flipMapKern.createMeanKernel(5,5,1);
	qImageProcessing::filterStackWithKernel(FlipMap,flipMapKern, qImageProcessing::OMIT_ZEROS, qImageProcessing::KERNEL_FILTER);
	if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;

	if( !qImage::exportImageFolder( FolderFlipMaps, "FlipMap_Slice", ".ima", FlipMap, FolderHigh) )
		return 1;



	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare T1 reconstruction
	if( loadT1 ) {
		if (!qImage::importImageFolder<qImageRaw>(FolderT1Maps, ".ima", nSlices,
				T1Map))
			return -1;
	}
	else {
		if (dumpGlobalInfo) GlobalInfoFile << "Start T1 Reconstruction ..."<<std::endl;
		greHighFlip.sort(sort_by_slice);
	    T1Recon.addRelevantData(greLowFlip,greHighFlip,FlipMap,nSlices);

		if( !T1Recon.run() ) {
			std::cout << "T1* Reconstruction Failed!" << std::endl;
			std::cout << T1Recon.getErrorMsg() << std::endl;
			exit(-1);
		}
		if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;

		T1Map=T1Recon.getT1Map();
		if( !qImage::exportImageFolder( FolderT1Maps, "T1Map_Slice", ".ima", T1Map, FolderHigh) )
			return 1;
	}


	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare H2O reconstruction
	if (dumpGlobalInfo) GlobalInfoFile << "Start H2O Reconstruction ..."<<std::endl;
	if( loadH2O ) {
		if (!qImage::importImageFolder<qImageRaw>(FolderH2OMaps, ".ima", nSlices,
				H2OMap))
			return -1;
	}
	else {

		// Calculate preliminary CSF mask (here we have to re-use the T1 map twice as no H2O map is
		// available at this stage)
		qImageProcessing::getBinaryMask(T1Map, T1Map, T2StarMap, CSFMask, BrainMask,
				1300,5000,80,3000,1300,5000,false);
		epiBody.sort(sort_by_slice);

	    H2ORecon.addRelevantData(T1Map,M0Map,FlipMap, CSFMask, epiBody, epiHigh,
	    		SNRMaskLowRes, FlipAngleGRELow, TRGRELow, nSlices);

		if( !H2ORecon.run() ) {
			std::cout << "H2O Reconstruction Failed!" << std::endl;
			std::cout << H2ORecon.getErrorMsg() << std::endl;
			exit(-1);
		}

		H2OMap=H2ORecon.getH2OMap();
		std::list<boost::shared_ptr<qImage> > CCMap = H2ORecon.getCoilCor();
	}



	// Check if High-B-Correction is required
	if( highFieldCorrection ) {

		// Calculate stringent masks for WM, GM, CSF --> required for unbiased estimate of median
		qImageProcessing::getBrainMask(T1Map, H2OMap, T2StarMap, BrainMask, true);
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, WMMask, BrainMask,
				500,850,35,60,500,1000, true);
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, GMMask, BrainMask,
				950,1300,35,70,500,1000, true);
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, CSFMask, BrainMask,
				1301,5000,80,5000,800,3500,true);

		double WMMedian= qImageStats::getStatistics(H2OMap,WMMask).getMedian(20,40);
		double GMMedian= qImageStats::getStatistics(H2OMap,GMMask).getMedian(20,40);
		double CSFMedian= qImageStats::getStatistics(H2OMap,CSFMask).getMedian(20,40);


		// .... now get less restrictive masks
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, WMMask, BrainMask,
				500,850,15,60,500,1000, false);
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, GMMask, BrainMask,
				950,1300,15,70,500,1000, false);
		qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, CSFMask, BrainMask,
				1301,5000,80,5000,800,3500,false);

		H2ORecon.calculateHighFieldCorrection(WMMask, GMMask, CSFMask, WMMedian, GMMedian, CSFMedian);
		H2ORecon.highFieldCorON();

		// Re-calculate H2O content, now with High-B correction
		if( !H2ORecon.run() ) {
			std::cout << "H2O Reconstruction Failed!" << std::endl;
			std::cout << H2ORecon.getErrorMsg() << std::endl;
			exit(-1);
		}
	}

	if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;

	// Store H2O Maps
	if( !qImage::exportImageFolder( FolderH2OMaps, "H2OMap_Slice", ".ima", H2OMap, FolderHigh) )
		return 1;

	std::list<boost::shared_ptr<qImage> > HighBCorMap = H2ORecon.getHighFieldCor();
	if( !qImage::exportImageFolder( FolderHighBCor, "HighBCor_Slice", ".ima", HighBCorMap, FolderHigh) )
		return 1;

	std::list<boost::shared_ptr<qImage> > CoilCorMap = H2ORecon.getCoilCor();
	if( !qImage::exportImageFolder( FolderCoilCor, "CoilCor_Slice", ".ima", CoilCorMap, FolderHigh) )
		return 1;



	////////////////////////////////////////////////////////////////////////////////////////////
	//prepare MEYLIN reconstruction
	if (dumpGlobalInfo) GlobalInfoFile << "Start Myelin Reconstruction ..."<<std::endl;
	if( loadMyelin ) {
		if (!qImage::importImageFolder<qImageRaw>(FolderMyelinMaps, ".ima", nSlices,
				MyelinMap))
			return -1;
	}
	else {
		MyelinRecon.addRelevantData(greLowFlip, T2StarMap, nSlices, SNR);

		// run myelin reconstruction
		if (!MyelinRecon.run()) {
			std::cout << "Myelin Reconstruction Failed!" << std::endl;
			std::cout << MyelinRecon.getErrorMsg() << std::endl;
			exit(-1);
		}

		if (dumpGlobalInfo) GlobalInfoFile << "...done" << std::endl;

		MyelinMap = MyelinRecon.getMyelinMap();
		T2StarFastMap = MyelinRecon.getT2StarFastMap();
		T2StarSlowMap = MyelinRecon.getT2StarSlowMap();

		if( !qImage::exportImageFolder( FolderMyelinMaps, "MyelinMap_Slice", ".ima", MyelinMap, FolderHigh) )
			return 1;
		if( !qImage::exportImageFolder( FolderT2StarSMaps, "T2Star(slow)_Slice", ".ima", T2StarSlowMap, FolderHigh) )
			return 1;
		if( !qImage::exportImageFolder( FolderT2StarFMaps, "T2Star(fast)_Slice", ".ima", T2StarFastMap, FolderHigh) )
			return 1;



		// Smooth myelin maps in z-Direction
		FilterKernel zSmoothKern;
		zSmoothKern.createEmptyKernel(1,1,2);
		zSmoothKern.setKernelValue(0,0,0,0.5);
		zSmoothKern.setKernelValue(0,0,1,0.5);
		std::list<boost::shared_ptr<qImage> > MyelinMapsSmoothed=qImageUtils::createEmptyImageList<qImageMap>(nRow, nCol, nSlices);
		qImageProcessing::sliceAverage(MyelinMap, MyelinMapsSmoothed, zSmoothKern);
		if( !qImage::exportImageFolder( FolderMyelinMapsSmooth, "MyelinMapSmoothed_Slice", ".ima", MyelinMapsSmoothed, FolderHigh) )
			return 1;
	}

	time(&now);
	if (dumpGlobalInfo) GlobalInfoFile << "Patient recon now ... finished at: "<<asctime(localtime(&now)) << std::endl;


	////////////////////////////////////////////////////////////////////////////////////////////
	//create SYNTHETIC images
	std::list<boost::shared_ptr<qImage> > FLASH_T1w=qSynthetic::calcFLASHImage(T1Map, T2StarMap, H2OMap,
			SNRMask, 5, 2.5, 90);
	if( !qImage::exportImageFolder( FolderSyntheticT1wGE, "T1wFLASH_Slice", ".ima", FLASH_T1w, FolderHigh) )
		return 1;



	////////////////////////////////////////////////////////////////////////////////////////////
	//SUMMARY STATISTICS

	// Calculate (final) masks for WM, GM, CSF and whole brain
	qImageProcessing::getBrainMask(T1Map, H2OMap, T2StarMap, BrainMask, true);
	qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, WMMask, BrainMask,
			500,850,35,60,500,1000, true);
	qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, GMMask, BrainMask,
			950,1400,35,70,500,1000, true);
	qImageProcessing::getBinaryMask(T1Map, H2OMap, T2StarMap, CSFMask, BrainMask,
			1401,5000,80,5000,800,3500,true);


	// get image statistics
	qImageStats WMH2OStats= qImageStats::getStatistics(H2OMap,WMMask);
	qImageStats WMT1Stats= qImageStats::getStatistics(T1Map,WMMask);
	qImageStats WMT2StarStats= qImageStats::getStatistics(T2StarMap,WMMask);
	qImageStats WMMyelinStats= qImageStats::getStatistics(MyelinMap,WMMask);

	qImageStats GMH2OStats= qImageStats::getStatistics(H2OMap,GMMask);
	qImageStats GMT1Stats= qImageStats::getStatistics(T1Map,GMMask);
	qImageStats GMT2StarStats= qImageStats::getStatistics(T2StarMap,GMMask);
	qImageStats GMMyelinStats= qImageStats::getStatistics(MyelinMap,GMMask);

	if (dumpGlobalInfo) {
		GlobalInfoFile << "SUMMARY STATISTICS for Dataset : " << PatientName << std::endl;
		GlobalInfoFile << "Gender : "<<PatientGender << std::endl;
		GlobalInfoFile << "Age:     "<<PatientAge << std::endl;
		GlobalInfoFile << "White Matter Averages:" <<std::endl;
		GlobalInfoFile << "    - T1     = "<<WMT1Stats.getMedian(20,40)<<" [ms]"<<std::endl;
		GlobalInfoFile << "    - T2*    = "<<WMT2StarStats.getMedian(20,40)<<" [ms]"<<std::endl;
		GlobalInfoFile << "    - H2O    = "<<WMH2OStats.getMedian(20,40)/10.0<<" [%]"<<std::endl;
		GlobalInfoFile << "    - Myelin = "<<WMMyelinStats.getMean(20,40)/10.0<<" / "
				<< WMMyelinStats.getMedian(20,40)/10.0<<" [%]"<<std::endl;
		GlobalInfoFile << "Grey Matter Averages:" <<std::endl;
		GlobalInfoFile << "    - T1     = "<<GMT1Stats.getMedian(20,40)<<" [ms]"<<std::endl;
		GlobalInfoFile << "    - T2*    = "<<GMT2StarStats.getMedian(20,40)<<" [ms]"<<std::endl;
		GlobalInfoFile << "    - H2O    = "<<GMH2OStats.getMedian(20,40)/10.0<<" [%]"<<std::endl;
		GlobalInfoFile << "    - Myelin = "<<GMMyelinStats.getMean(20,40)/10.0<<" / "
				<< GMMyelinStats.getMedian(20,40)/10.0<<" [%]"<<std::endl;
	}

	if( dumpGlobalRes ) {
		GlobalResFile << PatientAge.substr(0,3) <<" ";
		if (PatientGender.compare(0,1,"F")==0)
			GlobalResFile << 2 <<" ";
		else
			GlobalResFile << 1 <<" ";
		GlobalResFile <<WMT1Stats.getMedian(20,40)<<" ";
		GlobalResFile <<WMT2StarStats.getMedian(20,40)<<" ";
		GlobalResFile <<WMH2OStats.getMedian(20,40)/10.0<<" ";
		GlobalResFile <<WMMyelinStats.getMean(20,40)/10.0<<" ";

		GlobalResFile <<GMT1Stats.getMedian(20,40)<<" ";
		GlobalResFile <<GMT2StarStats.getMedian(20,40)<<" ";
		GlobalResFile <<GMH2OStats.getMedian(20,40)/10.0<<" ";
		GlobalResFile <<GMMyelinStats.getMean(20,40)/10.0<<" ";

		GlobalResFile << std::endl;
	}

	if( !qImage::exportImageFolder( FolderBrainMask, "BrainMask_Slice", ".ima", BrainMask, FolderHigh) ) return 1;
	if( !qImage::exportImageFolder( FolderWMMask, "WMMask_Slice", ".ima", WMMask, FolderHigh) ) return 1;
	if( !qImage::exportImageFolder( FolderGMMask, "GMMask_Slice", ".ima", GMMask, FolderHigh) ) return 1;
	if( !qImage::exportImageFolder( FolderCSFMask, "CSFMask_Slice", ".ima", CSFMask, FolderHigh) ) return 1;


	return 0;
}

