//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageBinMask.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageBinMask.cpp
 *
 *  Created on: 06.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "qImageBinMask.h"
#include <math.h>
#include <boost/numeric/ublas/matrix.hpp>

using namespace boost::numeric::ublas;


void StructElem::createDiskSE(double radius, int lengthX, int lengthY, int lengthZ) {
	boost::multi_array<double, 3>::extent_gen extents;
	SE.resize(extents[lengthX][lengthY][lengthZ]);

	// Assign member variables
	sizeX=lengthX;
	sizeY=lengthY;
	sizeZ=lengthZ;
	HotSpotX=(int)((sizeX+1)/2-1);
	HotSpotY=(int)((sizeY+1)/2-1);
	HotSpotZ=(int)((sizeZ+1)/2-1);

	for (boost::multi_array<double, 3>::index z = 0; z != lengthZ; ++z)
		for (boost::multi_array<double, 3>::index y = 0; y != lengthY; ++y)
			for (boost::multi_array<double, 3>::index x = 0; x != lengthX; ++x) {
				if( sqrt(pow(z-HotSpotZ,2)+pow(y-HotSpotY,2)+pow(x-HotSpotX,2)) <= radius )
					SE[x][y][z]=1;
				else
					SE[x][y][z]=0;
			}
}



qImageBinMask::qImageBinMask() {
	// TODO Auto-generated constructor stub

}

qImageBinMask::qImageBinMask(int nrow=0, int ncol=0) : qImage(nrow, ncol) {
}


qImageBinMask::qImageBinMask(vector<int> &dimIdx, int nrow=0, int ncol=0) : qImage(dimIdx, nrow, ncol) {
}


qImageBinMask::~qImageBinMask() {
	// TODO Auto-generated destructor stub
}


void qImageBinMask::AND(qImage &secondMask) {
	if( !checkSize(secondMask) )
		throw "qImageBinMask::AND : incompatible image size";

	matrix<int> secData=secondMask.getPixelMatrix();
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {
			if(pixel(row,col) > 0 && secData(row,col) > 0)
				pixel(row,col)=1;
			else
				pixel(row,col)=0;
		}
	}
}


void qImageBinMask::OR(qImage &secondMask) {
	if( !checkSize(secondMask) )
		throw "qImageBinMask::OR : incompatible image size";

	matrix<int> secData=secondMask.getPixelMatrix();
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {
			if(pixel(row,col) > 0 || secData(row,col) > 0)
				pixel(row,col)=1;
			else
				pixel(row,col)=0;
		}
	}
}


void qImageBinMask::invert() {
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {
			if(pixel(row,col) > 0 )
				pixel(row,col)=0;
			else
				pixel(row,col)=1;
		}
	}
}


void qImageBinMask::dilate(StructElem SE) {

	// make (deep) pixel copy
	matrix<int> pixelCopy = pixel;

	// get extension of the SE
	int extX= (int)(SE.getSizeX()/2);
	int extY= (int)(SE.getSizeY()/2);

	// loop over all points and apply SE
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {

			if( pixelCopy(row,col) == 0)
				continue;

			for(int SEcol=-extX; SEcol<extX; SEcol++)
				for(int SErow=-extY; SErow<extY; SErow++)
					if( row+SErow >= 0 && row+SErow<nRow &&
							col+SEcol >= 0 && col+SEcol<nCol &&
							SE.getElement(SEcol,SErow,0) > 0 ) {
						pixel(row+SErow, col+SEcol) = 1;
					}
		}
	}
}



void qImageBinMask::erode(StructElem SE) {

	// make (deep) pixel copy
	matrix<int> pixelCopy = pixel;

	// get extension of the SE
	int extX= (int)(SE.getSizeX()/2);
	int extY= (int)(SE.getSizeY()/2);

	// loop over all points and apply SE
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {

			if( pixelCopy(row,col) == 0)
				continue;

			for(int SEcol=-extX; SEcol<extX; SEcol++)
				for(int SErow=-extY; SErow<extY; SErow++)
					if( SE.getElement(SEcol,SErow,0) > 0 ) {
						if(row+SErow < 0 || row+SErow>=nRow || col+SEcol < 0 || col+SEcol>=nCol)  {
							pixel(row, col) = 0;
							break;
						} else if( pixelCopy(row+SErow,col+SEcol) == 0) {
							pixel(row, col) = 0;
							break;
						}
					}
		}
	}
}



boost::shared_ptr<qImage> qImageBinMask::label(int &labelCount) {

	boost::shared_ptr<qImage> labelImage = boost::shared_ptr<qImage>(new qImage(nRow, nCol));
	matrix<int> labelPixel = labelImage->getPixelMatrix();

	int minLabel, maxLabel=0;
    int leftLabel, upperLabel = 0;
    labelCount = 1;

	// loop over all points and apply SE
	for(int row=0; row<nRow; row++) {
		for(int col=0; col<nCol; col++) {

			if( pixel(row,col) == 0)
				continue;

            // default lables
            leftLabel = -1;
            upperLabel = -1;

            // left and upper neighbour
            if (col > 0) leftLabel = labelPixel(row,col-1);
            if (row > 0) upperLabel = labelPixel(row-1,col);

            if (leftLabel <= 0 && upperLabel <= 0)
            	labelPixel(row,col)=labelCount++;
           	else if (leftLabel <= 0 && upperLabel > 0)
            	labelPixel(row,col)=upperLabel;
            else if (leftLabel > 0 && upperLabel <= 0)
            	labelPixel(row,col)=leftLabel;
            else {
            	 minLabel=(int)std::min(upperLabel,leftLabel);
                 maxLabel=(int)std::max(upperLabel,leftLabel);

            	labelPixel(row,col)=minLabel;

            	// Collision
                if (leftLabel != upperLabel) {
                    for( int ccol=0; ccol<nCol; ccol++ )
                        for( int crow=0; crow<=row; crow++ )
                            if( labelPixel(crow, ccol)==maxLabel )
                            	labelPixel(crow,ccol)=minLabel;
                            else if( labelPixel(crow, ccol)>=maxLabel )
                            	labelPixel(crow,ccol)--;
                    labelCount-=1;
                }
            }
		}
	}

	labelImage->setPixelMatrix(labelPixel);
	return labelImage;
}

