//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageBinMask.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageBinMask.h
 *
 *  Created on: 06.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#ifndef QIMAGEBINMASK_H_
#define QIMAGEBINMASK_H_

#include "qImage.h"
#include "boost/multi_array.hpp"

class StructElem {

	int sizeX;
	int sizeY;
	int sizeZ;
	int HotSpotX;
	int HotSpotY;
	int HotSpotZ;

	boost::multi_array<double, 3> SE;

public:
	StructElem() {};
	virtual ~StructElem() {};
	void createDiskSE(double radius, int sizeX, int sizeY, int sizeZ);

	double getElement(int xrel, int yrel, int zrel) {
		return SE[xrel+HotSpotX][yrel+HotSpotY][zrel+HotSpotZ];
	};

	int getSizeX() { return sizeX; };
	int getSizeY() { return sizeY; };
	int getSizeZ() { return sizeZ; };
};



class qImageBinMask: public qImage {
public:
	qImageBinMask();
	qImageBinMask(int nrow, int ncol);
	qImageBinMask(vector<int> &dimensionIndex, int nrow, int ncol);
	virtual ~qImageBinMask();

	void AND(qImage &secondMask);
	void OR(qImage &secondMask);

	void invert();
	void dilate(StructElem SE);
	void erode(StructElem SE);
	void fillHoles(StructElem SE) { dilate(SE); erode(SE); };

	boost::shared_ptr<qImage> label(int &labelCount);
};

#endif /* QIMAGEBINMASK_H_ */
