//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageProcessing.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageProcessing.h
 *
 *  Created on: 28.01.2012
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#ifndef QIMAGEPROCESSING_H_
#define QIMAGEPROCESSING_H_

#include "boost/multi_array.hpp"
#include <boost/numeric/ublas/matrix.hpp>
using namespace boost::numeric::ublas;

class FilterKernel {
	int sizeX;
	int sizeY;
	int sizeZ;
	double kernNorm;
	boost::multi_array<double, 3> kernel;

public:
	FilterKernel() {};
	virtual ~FilterKernel() {};

	double applyFilter(matrix<int>& Image, int x, int y);
	double applyFilter(boost::multi_array<int, 3>& Image, int x, int y, int z);

	double getKernValue(int x, int y, int z) {
		return kernel[y][x][z];
	};

	double getKernValueUnnorm(int x, int y, int z) {
		return kernel[y][x][z] / kernNorm;
	};

	void createMeanKernel(int kernX, int kernY, int kernZ);
	void createEmptyKernel(int kernX, int kernY, int kernZ);
	void setKernelValue(int X, int Y, int Z, double value) { kernel[Y][X][Z]=value; };

	int getKernSizeX() {
		return sizeX;
	};

	int getKernSizeY() {
		return sizeY;
	};

	int getKernSizeZ() {
		return sizeZ;
	};
};




class qImageProcessing {
public:

	enum FILTER_OPTIONS {
		NONE, OMIT_ZEROS, FILL_ZEROS_ONLY
	};

	enum FILTER_TYPE {
		KERNEL_FILTER, MEDIAN_FILTER, REMOVE_OUTLIER
	};

	enum PHASE_ENCODE_DIRECTION {
		ROW, COL
	};

	qImageProcessing();
	virtual ~qImageProcessing();

	static void filterStackWithKernel(
			std::list<boost::shared_ptr<qImage> >& image, FilterKernel &kernel,
			FILTER_OPTIONS options, FILTER_TYPE tpye);

	static void filterWithKernel(boost::shared_ptr<qImage> & image,
			FilterKernel &kernel, FILTER_OPTIONS options, FILTER_TYPE filterTpye,
			double outlierCutoff=0.2);

	static void sliceAverage(std::list<boost::shared_ptr<qImage> >& original,
			std::list<boost::shared_ptr<qImage> >& copy, FilterKernel &kernel);

	static void getBrainMask(std::list<boost::shared_ptr<qImage> >& T1Map,
			std::list<boost::shared_ptr<qImage> >& H2OMap,
			std::list<boost::shared_ptr<qImage> >& T2StarMap,
			std::list<boost::shared_ptr<qImage> >& BrainMask,
			bool useH2O);

	static void getBinaryMask(std::list<boost::shared_ptr<qImage> >& T1Map,
			std::list<boost::shared_ptr<qImage> >& H2OMap,
			std::list<boost::shared_ptr<qImage> >& T2StarMap,
			std::list<boost::shared_ptr<qImage> >& mask,
			std::list<boost::shared_ptr<qImage> >& superMask,
			int T1low, int T1high, int T2low, int T2high, int H2Olow, int H2Ohigh,
			bool useSuperMask);

	static void correctGRELowFlipArtifact(std::list<boost::shared_ptr<qImage> >& GRELowFlip,
			std::list<boost::shared_ptr<qImage> >& T2StarMap,
			std::list<boost::shared_ptr<qImage> >& mask);

	static void motionCorrection(std::list<boost::shared_ptr<qImage> >& magnitude,
			std::list<boost::shared_ptr<qImage> >& phase,
			std::list<boost::shared_ptr<qImage> >& mask,
			PHASE_ENCODE_DIRECTION peDirection,
			int nEchoTimes=1
			);

};

#endif /* QIMAGEPROCESSING_H_ */
