//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageStats.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageStats.cpp
 *
 *  Created on: 11.03.2012
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "qImageStats.h"
#include "qImageBinMask.h"
#include <list>


qImageStats::qImageStats() {
	// TODO Auto-generated constructor stub

}

qImageStats::~qImageStats() {
	// TODO Auto-generated destructor stub
}


qImageStats qImageStats::getStatistics(std::list<boost::shared_ptr<qImage> > imageStack,
									  std::list<boost::shared_ptr<qImage> > mask) {

	qImageStats newStats;
	std::list< boost::shared_ptr<qImage> >::iterator maskIter;
	std::list< boost::shared_ptr<qImage> >::iterator imageIter;
	std::vector<int> greyValues;
	std::vector<double> sliceMean;
	std::vector<double> sliceMedian;
	std::vector<double> sliceVolume;

	int meanSum=0;
	int GWCount = 0;
	double mean=0;
	double median=0;

	// determine stack size
	int nSlices=imageStack.size();
	int nRow=(*imageStack.begin())->getNrows();
	int nCol=(*imageStack.begin())->getNcols();

	// loop over all points
	// Loop over all slices
	for( int slice=1; slice <= nSlices; slice++) {

		double meanSumSlice=0;
		double GWCountSlice=0;
		double meanSlice=0;
		double medianSlice=0;
		std::vector<int> greyValuesSlice;

		// set iterators properly (required for parallel processing)
		imageIter = imageStack.begin(); std::advance(imageIter,slice-1);
		maskIter = mask.begin(); std::advance(maskIter,slice-1);

		// get all images (timepoints) for current slice
		matrix<int>& maskMat = (*maskIter)->getPixelMatrix();
		matrix<int>& imageMat = (*imageIter)->getPixelMatrix();

		// loop over all points in matrix
		for(int row=0; row<nRow; row++) {
			for(int col=0; col<nCol; col++) {

				// is current pixel contained in image mask?
				if( maskMat(row,col) == 0)
					continue;

				greyValues.push_back(imageMat(row,col));
				greyValuesSlice.push_back(imageMat(row,col));
				meanSum += imageMat(row,col);
				meanSumSlice += imageMat(row,col);
				GWCount++; GWCountSlice++;
			}
		}

		if( GWCountSlice > 0 ) {
			// sort vector and take median value
			std::sort(greyValuesSlice.begin(),greyValuesSlice.end());
			medianSlice = greyValuesSlice[(int)( GWCountSlice/2.0 )];
			meanSlice = meanSumSlice/(double)GWCountSlice;
		}

		sliceMean.push_back(meanSlice);
		sliceMedian.push_back(medianSlice);
		sliceVolume.push_back(GWCountSlice);
	}

	if( GWCount > 0 ) {
		// sort vector and take median value
		std::sort(greyValues.begin(),greyValues.end());
		median = greyValues[(int)( GWCount/2.0 )];
		mean = meanSum/(double)GWCount;
	}

	newStats.mean=mean;
	newStats.median=median;
	newStats.volume=GWCount;
	newStats.sliceMean=sliceMean;
	newStats.sliceMedian=sliceMedian;
	newStats.sliceVolume=sliceMedian;

	return newStats;
}


double qImageStats::getMean(int firstSlice, int lastSlice) {

	if( firstSlice == lastSlice )
		return mean;

	double weightedSum=0;
	double normSum=0;

	for(int slice=firstSlice-1; slice<lastSlice; slice++) {
		weightedSum += sliceMean[slice]*sliceVolume[slice];
		normSum += sliceVolume[slice];
	}

	if( normSum != 0 )
		return weightedSum/normSum;
	else
		return 0;
}


double qImageStats::getMedian(int firstSlice, int lastSlice) {

	if( firstSlice == lastSlice )
		return median;

	double weightedSum=0;
	double normSum=0;

	for(int slice=firstSlice-1; slice<lastSlice; slice++) {
		weightedSum += sliceMedian[slice]*sliceVolume[slice];
		normSum += sliceVolume[slice];
	}

	if( normSum != 0 )
		return weightedSum/normSum;
	else
		return 0;
}

double qImageStats::getVolume(int firstSlice, int lastSlice) {

	if( firstSlice == lastSlice )
		return volume;

	double volSum=0;

	for(int slice=firstSlice-1; slice<lastSlice; slice++)
		volSum += sliceVolume[slice];

	return volSum;
}

