//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageStats.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageStats.h
 *
 *  Created on: 11.03.2012
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#ifndef QIMAGESTATS_H_
#define QIMAGESTATS_H_

#include "qImage.h"
#include <boost/shared_ptr.hpp>

class qImageStats {

	double median;
	double mean;
	double volume;
	std::vector<double> sliceMean;
	std::vector<double> sliceMedian;
	std::vector<double> sliceVolume;

public:
	qImageStats();
	virtual ~qImageStats();

	static qImageStats getStatistics(std::list<boost::shared_ptr<qImage> > imageStack,
			std::list<boost::shared_ptr<qImage> > mask);

	double getMean(int firstSlice=0, int lastSlice=0);
	double getMedian(int firstSlice=0, int lastSlice=0);
	double getVolume(int firstSlice=0, int lastSlice=0);

	std::vector<double> getSliceMean() { return sliceMean; };
	std::vector<double> getSliceMedian() { return sliceMedian; };
	std::vector<double> getSliceVolume() { return sliceVolume; };
	double getSliceMean(int slice) { return sliceMean[slice]; };
	double getSliceMedian(int slice) { return sliceMedian[slice]; };
	double getSliceVolume(int slice) { return sliceVolume[slice]; };
};

#endif /* QIMAGESTATS_H_ */
