//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageUtils.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageUtils.cpp
 *
 *  Created on: 07.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "qImageUtils.h"
#include "qImage.h"
#include <boost/numeric/ublas/io.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <iostream>


using namespace boost::numeric::ublas;


qImageUtils::qImageUtils() {
	// TODO Auto-generated constructor stub

}

qImageUtils::~qImageUtils() {
	// TODO Auto-generated destructor stub
}


double qImageUtils::getNoiseLevel(std::list< boost::shared_ptr<qImage> > &rawImage) {

	double noiseLevel=0;
	double count=0;
	std::list< boost::shared_ptr<qImage> >::iterator iter;

	for(iter=rawImage.begin(); iter!=rawImage.end(); iter++) {
		matrix_range<matrix<int> > mr ((*iter)->getPixelMatrix(), range (1, 5), range (1, 5));
	    matrix<int> mrange=mr;

	    noiseLevel+=norm_1(mr);
	    count+=mr.size1()*mr.size2();
	}

	return noiseLevel/count;
}



double qImageUtils::createSNRMask(double cutoff,
					        	  std::list< boost::shared_ptr<qImage> > &rawImage,
		                          std::list< boost::shared_ptr<qImage> > &SNRMask) {

	double signal=0;
	int count=0;
	boost::shared_ptr<qImage> curImg;
	std::list< boost::shared_ptr<qImage> >::iterator iter;

	// get SNR
	double Noise = getNoiseLevel(rawImage);

	// loop over all points and set mask
	for(iter=rawImage.begin(); iter!=rawImage.end(); iter++) {
		// create new mask object (slice)
		curImg = boost::shared_ptr<qImageBinMask>(new qImageBinMask((*iter)->getDimensionIdx(), (*iter)->getNrows(), (*iter)->getNcols()));
		SNRMask.push_back(curImg);

		matrix<int> rawMat=(*iter)->getPixelMatrix();
		matrix<int>& maskMat=curImg->getPixelMatrix();

		for(int row=0; row<(*iter)->getNrows(); row++) {
			for( int col=0; col<(*iter)->getNcols(); col++) {
				if( rawMat(row,col) < cutoff*Noise )
					maskMat(row,col) = 0;
				else {
					maskMat(row,col) = 1;
					signal += rawMat(row,col);
					count++;
				}
			}
		}
	}

	return signal/Noise/count;
}


std::list< boost::shared_ptr<qImage> > qImageUtils::getImagesFromSlice(std::list< boost::shared_ptr<qImage> > fullList,
		                                                               int slice) {

	std::list< boost::shared_ptr<qImage> >::iterator iter;
	std::list< boost::shared_ptr<qImage> > sliceList;

	for(iter=fullList.begin(); iter!=fullList.end(); iter++)
		if( (*iter)->getDimensionIdx()(2) == slice )
			sliceList.push_back(*iter);

	return sliceList;

}


void qImageUtils::getImageArray_DeepCopy(std::list< boost::shared_ptr<qImage> > imageList,
		boost::multi_array<int, 3> &imageArray) {

    std::list<boost::shared_ptr<qImage> >::iterator imageIter;

	// get image and kernel dimensions
	int nSlices=imageList.size();
	int nRows=(*imageList.begin())->getNrows();
	int nCols=(*imageList.begin())->getNcols();

	boost::multi_array<int, 3>::extent_gen extents;
	imageArray.resize(extents[nRows][nCols][nSlices]);

	// Loop over all points and perform deep copy
	for (int slice = 0; slice < nSlices; slice++) {

		// set iterators and get matrix
		imageIter = imageList.begin();
		std::advance(imageIter, slice);
		matrix<int>& imageMat = (*imageIter)->getPixelMatrix();

		for(int row=0; row<nRows; row++) {
			for( int col=0; col<nCols; col++) {
				imageArray[row][col][slice]=imageMat(row,col);
			}
		}
	}
}




bool sort_by_slice(boost::shared_ptr<qImage> first, boost::shared_ptr<qImage> second)
{
	vector<int> firstVec=first->getDimensionIdx();
	vector<int> secVec=second->getDimensionIdx();

	if(firstVec(2) < secVec(2)) return true;
	else return false;
}
