//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qImageUtils.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qImageUtils.h
 *
 *  Created on: 07.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include <list>
#include <boost/shared_ptr.hpp>
#include "boost/multi_array.hpp"
#include "qImage.h"
#include "qImageBinMask.h"

#ifndef QIMAGEUTILS_H_
#define QIMAGEUTILS_H_

class qImageUtils {
private:
	qImageUtils();
	virtual ~qImageUtils();

public:

	static double getNoiseLevel(std::list< boost::shared_ptr<qImage> > &rawImage);

	static double createSNRMask(double cutoff,
				  			    std::list< boost::shared_ptr<qImage> > &RawImage,
			                    std::list< boost::shared_ptr<qImage> > &SNRMask);

	static std::list< boost::shared_ptr<qImage> > getImagesFromSlice(std::list< boost::shared_ptr<qImage> > fullList,
			                                                         int slice);

	static void getImageArray_DeepCopy(std::list< boost::shared_ptr<qImage> > imageList,
								       boost::multi_array<int, 3> &imageArray);

	template<typename T> static std::list< boost::shared_ptr<qImage> > getImageList_DeepCopy(std::list< boost::shared_ptr<qImage> > original) {

		std::list< boost::shared_ptr<qImage> > copy;
	    std::list<boost::shared_ptr<qImage> >::iterator imageIter;
	    boost::shared_ptr<qImage> curImg;

		// get image and kernel dimensions
		int nSlices=original.size();

		// Loop over all points and perform deep copy
		for (int slice = 0; slice < nSlices; slice++) {

			// set iterators and get matrix
			imageIter = original.begin();
			std::advance(imageIter, slice);
//			curImg = boost::shared_ptr<qImage>(new T(**imageIter));
			curImg = boost::shared_ptr<qImage>(new T(*boost::static_pointer_cast<T>(*imageIter)));
			copy.push_back(curImg);
		}

		return copy;
	}


	template<typename T> static std::list< boost::shared_ptr<qImage> > createEmptyImageList(int nrow, int ncol, int nslices) {

		std::list< boost::shared_ptr<qImage> > newList;
		boost::shared_ptr<qImage> newImage;

		for(int slice=0; slice<nslices; slice++) {
			newImage = boost::shared_ptr<T>(new T(nrow, ncol));
			newList.push_back(newImage);
		}
		return newList;
	}


	template<typename T> static boost::shared_ptr<qImage>  resize(int nColsNew, int nRowsNew,
			boost::shared_ptr<qImage> Original) {

		// get original size
		int nColsOrig = Original->getNcols();
		int nRowsOrig = Original->getNrows();

		// get scaling factors
		float scaleRow=nRowsOrig/(float)nRowsNew;
		float scaleCol=nColsOrig/(float)nColsNew;

		// create new image object
		boost::shared_ptr<T> curImg=boost::shared_ptr<T>(new T(nRowsNew, nColsNew));
		curImg->setHeader(Original->getHeader());

		// copy with nearest neighbour interpolation
		matrix<int>& origMat = Original->getPixelMatrix();
		for(int row=0; row<nRowsNew; row++) {
			for(int col=0; col<nColsNew; col++) {
				int value=origMat( (int)(row*scaleRow), (int)(col*scaleCol) );
				curImg->setPixel(row,col,value);
			}
		}

		return curImg;
	}



	template<typename T> static std::list< boost::shared_ptr<qImage> > resize(int nColsNew, int nRowsNew,
				  									                          std::list< boost::shared_ptr<qImage> > Original) {
		std::list< boost::shared_ptr<qImage> > sliceList;
		std::list<boost::shared_ptr<qImage> >::iterator iter;
		boost::shared_ptr<qImage> newImg;

		// copy
		for(iter=Original.begin(); iter!=Original.end(); iter++) {
			newImg = resize<T>(nColsNew, nRowsNew, *iter);
			sliceList.push_back(newImg);
		}

		return sliceList;
	}

};


// Sort functions
bool sort_by_slice(boost::shared_ptr<qImage> first, boost::shared_ptr<qImage> second);


#endif /* QIMAGEUTILS_H_ */
