//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qSynthetic.cpp $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qSynthetic.cpp
 *
 *  Created on: 20.03.2012
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include "qSynthetic.h"
#include "qImageUtils.h"
#include "qImageMap.h"
#include <math.h>

qSynthetic::qSynthetic() {
	// TODO Auto-generated constructor stub

}

qSynthetic::~qSynthetic() {
	// TODO Auto-generated destructor stub
}


std::list<boost::shared_ptr<qImage> > qSynthetic::calcFLASHImage(std::list<boost::shared_ptr<qImage> > T1,
		std::list<boost::shared_ptr<qImage> > T2Star,
		std::list<boost::shared_ptr<qImage> > H2O,
		std::list<boost::shared_ptr<qImage> > mask,
		double TR,
		double TE,
		double alpha) {

	std::list< boost::shared_ptr<qImage> >::iterator T1Iter;
	std::list< boost::shared_ptr<qImage> >::iterator T2Iter;
	std::list< boost::shared_ptr<qImage> >::iterator H2OIter;
	std::list< boost::shared_ptr<qImage> >::iterator maskIter;
	std::list< boost::shared_ptr<qImage> >::iterator fLASHIter;

	int nSlices = T1.size();
	int nRow = (*T1.begin())->getNrows();
	int nCol = (*T1.begin())->getNcols();
	double alphaRAD=alpha/180.0*PI;

	std::list< boost::shared_ptr<qImage> > FLASH=qImageUtils::createEmptyImageList<qImageMap>(nRow, nCol, nSlices);

	// Loop over all slices
	for( int slice=0; slice < nSlices; slice++) {

		T1Iter = T1.begin(); std::advance(T1Iter,slice);
		T2Iter = T2Star.begin(); std::advance(T2Iter,slice);
		H2OIter = H2O.begin(); std::advance(H2OIter,slice);
		maskIter = mask.begin(); std::advance(maskIter,slice);
		fLASHIter = FLASH.begin(); std::advance(fLASHIter,slice);

		matrix<int>& maskMat = (*maskIter)->getPixelMatrix();
		matrix<int>& T1Mat = (*T1Iter)->getPixelMatrix();
		matrix<int>& T2Mat = (*T2Iter)->getPixelMatrix();
		matrix<int>& H2OMat = (*H2OIter)->getPixelMatrix();
		matrix<int>& fLASHMat = (*fLASHIter)->getPixelMatrix();

		// loop over all points in matrix
		for(int row=0; row<nRow; row++) {
			for(int col=0; col<nCol; col++) {

				// is current pixel contained in image mask?
				if( maskMat(row,col) == 0) {
					fLASHMat(row,col)=0;
					continue;
				}

				fLASHMat(row,col) = (int) ( 100*H2OMat(row,col) * exp(-TE/(double)T2Mat(row,col)) * sin( alphaRAD )
						* (1-exp(-TR/(double)T1Mat(row,col))) / (1-cos(alphaRAD)*exp(-TR/(double)T1Mat(row,col))) );
			}
		}
	}

	return FLASH;
}
