//  File name: $HeadURL: https://www.rheinahrcampus.de/svn/repos/predict4a/pLib/src/qublasUtils.h $
//  Revision: $Revision: 53 $
//  Last modified: $Date: 2012-05-02 10:46:15 +0200 (Mi, 02 Mai 2012) $
//  Last modified by: $Author: neeb $

/*
 * qublasUtils.h
 *
 *  Created on: 05.09.2011
 *      Author: neeb

	Copyright 2012 Heiko Neeb

    This file is part of PREDICT-MS (Professional Remagen Diagnostic Image Calculation Tool for MS).

    PREDICT-MS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PREDICT-MS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with "predict".  If not, see <http://www.gnu.org/licenses/>.

	PREDICT-MS is designed to be used for scientific purposes only. Do NOT
	base any diagnosis on the results obtained with this program

 	The software is based on the following work which the authors kindly ask to reference in
	any scientific work which is based on PREDICT-MS or parts of it:

	V. Tonkova, V. Arhelger, J. Schenk, H. Neeb. Rapid Myelin Water Content Mapping on Clinical MR Systems.
	Z Med Phys (2012): http://dx.doi.org/10.1016/j.zemedi.2011.09.005

	H. Neeb, V. Ermer, T. Stocker, N.J. Shah. Fast quantitative mapping of absolute water content
	with full brain coverage. NeuroImage (2008). 42(3): 10941109

 */

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include "qImage.h"
#include <iostream>
#include <fstream>
#include <istream>
#include <sstream>


#ifndef QUBLASUTILS_H_
#define QUBLASUTILS_H_

using namespace boost::numeric::ublas;


class qublasUtils {
private:

	// Copied from http://www.codeproject.com/Articles/21909/Introduction-to-dynamic-two-dimensional-arrays-in
	// on April, 15th, 2012
	template <typename T> static T **AllocateDynamicArray( int nCols, int nRows)
	{
	      T **dynamicArray;

	      dynamicArray = new T*[nCols];
	      for( int i = 0 ; i < nCols ; i++ )
	      dynamicArray[i] = new T [nRows];

	      return dynamicArray;
	}

	template <typename T> static void FreeDynamicArray(T** dArray)
	{
	      delete [] *dArray;
	      delete [] dArray;
	}


	qublasUtils();
	virtual ~qublasUtils();

public:

	static bool fillMatrixFromTextFile(std::string fName, matrix<int> &mat, int &rows, int &cols);
	static void linearFit(const vector<float> x, const vector<float> y, double &slope, double &offset);

	static void FFT2D(qImage fAbs, qImage fPhase, qImage& FAbs, qImage& FPhase, bool inverse=false);
	static void FFT2D(int n, int m, bool inverse, vector<double>& fReal, vector<double>& fImag, vector<double>& FReal, vector<double>& FImag);


	template<typename T> static bool storeMatrixInTextFile(std::string fName, matrix<T> &mat, int rows, int cols) {
		std::ofstream fout;
		fout.open(fName.c_str());
		if( !fout ) {
			std::cout << "can't open output file" << std::endl;
			return false;
		}

		if( rows==0 || cols == 0) {
			rows = mat.size1();
			cols = mat.size2();
		}

		for( int row=0; row<rows; row++ ) {
			for( int col=0; col<cols; col++ ) {
				fout << mat(row,col) << " ";
			}
			fout << std::endl;
		}

		fout.close();

		return true;

	}

	template<typename T> static bool storeVectorInTextFile(std::string fName, vector<T> &vec, int length) {
		std::ofstream fout;
		fout.open(fName.c_str());
		if( !fout ) {
			std::cout << "can't open output file" << std::endl;
			return false;
		}

		if( length==0 )
			length = vec.size();

		for( int idx=0; idx<length; idx++ )
			fout << vec(idx) << " ";
		fout << std::endl;

		fout.close();

		return true;
	}


	template<typename T> static void concat(vector<T>& first, const vector<T>& second) {
		int firstSz=first.size();
		int secSz=second.size();

		first.resize(firstSz+secSz,true);
		subrange(first, firstSz, firstSz+secSz) = second;
	};


	template<typename T> static void hconcat(matrix<T>& first, const matrix<T>& second) {
		int nRow=first.size1();
		int firstNCol=first.size2();
		int secNCol=second.size2();

		if( nRow != (int)second.size1() )
			throw "qublasUtils::hconcat(): inconsistent number of matrix rows";

		first.resize(nRow,firstNCol+secNCol,true);
		subrange(first, 0, nRow, firstNCol, firstNCol+secNCol) = second;
	};


	template<typename T> static void vconcat(matrix<T>& first, const matrix<T>& second) {
		int nCol=first.size2();
		int firstNRow=first.size1();
		int secNRow=second.size1();

		if( nCol != (int)second.size2() )
			throw "qublasUtils::vconcat(): inconsistent number of matrix columns";

		first.resize(firstNRow+secNRow,nCol,true);
		subrange(first, firstNRow, firstNRow+secNRow, 0, nCol) = second;
	};


	template<typename T> static void row_equals(int row, double value, const matrix<T>& orig, matrix<T>& res) {
		int nRow=orig.size1();
		int nCol=orig.size2();
		int count=0;
		float eps=value*0.00001;
		res = matrix<T>(nRow, nCol);

		for(int col=0; col<nCol; col++)
			if( fabs( orig(row,col)-value ) <eps)
				column(res,count++) = column(orig,col);

		res.resize(nRow, count, true);
	};


	template<typename T> static void col_equals(int col, double value, const matrix<T>& orig, matrix<T>& res) {
		int nRow=orig.size1();
		int nCol=orig.size2();
		int count=0;
		float eps=value*0.00001;
		res = matrix<T>(nRow, nCol);

		for(int crow=0; crow<nRow; crow++)
			if( fabs(orig(crow,col-1)-value) < eps)
				row(res,count++) = row(orig,crow);

		res.resize(count, nCol, true);
	};



	template<typename T> static int get_col_min_dist_idx(int col, double value, const matrix<T>& orig) {
		int nRow=orig.size1();
		T curMinDist=100000000.0;
		int shortestIdx = -1;
		float newMinDist = 0;
		for(int crow=0; crow<nRow; crow++) {
			newMinDist = fabs(orig(crow,col-1)-value);
			if( newMinDist < curMinDist) {
				curMinDist=newMinDist;
				shortestIdx=crow;
			}
		}
		return shortestIdx;
	}

};


#endif /* QUBLASUTILS_H_ */
